# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: designers.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='designers.proto',
  package='',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x0f\x64\x65signers.proto\"Q\n\x11\x44\x65signerInfoProto\x12\x10\n\x08\x64\x65signer\x18\x01 \x01(\t\x12\x0c\n\x04link\x18\x02 \x01(\t\x12\x1c\n\x06\x61vatar\x18\x03 \x01(\x0b\x32\x0c.AvatarProto\" \n\x0b\x41vatarProto\x12\x11\n\tfile_name\x18\x01 \x01(\t')
)




_DESIGNERINFOPROTO = _descriptor.Descriptor(
  name='DesignerInfoProto',
  full_name='DesignerInfoProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='designer', full_name='DesignerInfoProto.designer', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='link', full_name='DesignerInfoProto.link', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='avatar', full_name='DesignerInfoProto.avatar', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=19,
  serialized_end=100,
)


_AVATARPROTO = _descriptor.Descriptor(
  name='AvatarProto',
  full_name='AvatarProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_name', full_name='AvatarProto.file_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=102,
  serialized_end=134,
)

_DESIGNERINFOPROTO.fields_by_name['avatar'].message_type = _AVATARPROTO
DESCRIPTOR.message_types_by_name['DesignerInfoProto'] = _DESIGNERINFOPROTO
DESCRIPTOR.message_types_by_name['AvatarProto'] = _AVATARPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DesignerInfoProto = _reflection.GeneratedProtocolMessageType('DesignerInfoProto', (_message.Message,), dict(
  DESCRIPTOR = _DESIGNERINFOPROTO,
  __module__ = 'designers_pb2'
  # @@protoc_insertion_point(class_scope:DesignerInfoProto)
  ))
_sym_db.RegisterMessage(DesignerInfoProto)

AvatarProto = _reflection.GeneratedProtocolMessageType('AvatarProto', (_message.Message,), dict(
  DESCRIPTOR = _AVATARPROTO,
  __module__ = 'designers_pb2'
  # @@protoc_insertion_point(class_scope:AvatarProto)
  ))
_sym_db.RegisterMessage(AvatarProto)


# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
"""
Set ufo attributes for a collection of ufos

Usage:

# set openTypeOS2TypoAscender for a collection of ufos:
gftools ufo-setter --ufos ufo1.ufo ufo2.ufo --attribs info.openTypeOS2TypoAscender=700 

# set panose for a ufo:
gftools ufo-setter --ufos ufo1.ufo ufo2.ufo --attribs info.openTypeOS2Panose=[0,0,0,0,0,0,0,0,0,0]

# Set typo vertical metrics for a collection of ufos:
gftools ufo-setter --ufos ufo1.ufo ufo2.ufo ufo3.ufo --attribs \
    info.openTypeOS2TypoAscender=1000 \
    info.openTypeOS2TypoDescender=-250 \
    info.openTypeOS2TypoLineGap=0
"""
# TODO this could be generalised for all font formats
import argparse
from defcon import Font
import json


def parse_attributes(items):
    res = {}
    for item in items:
        k, v = item.split("=")
        res[k] = v
    return res


def set_ufo_attrib(obj, path, val):
    # use dfs to access nested attributes
    stack = [(obj, path)]
    while stack:
        item, path = stack.pop()
        if len(path) == 1:
            try:
                parsed_val = json.loads(val)
            except:
                parsed_val = val
            setattr(item, path[0], parsed_val)
            return
        current = path.pop(0)
        if hasattr(item, current):
            sub_item = getattr(item, current)
            stack.append((sub_item, path))
        elif current in item:
            sub_item = item[current]
            stack.append(sub_item, path)
        else:
            raise ValueError(f"{current} is not a valid attribute of {item}")
    return True


def main(args=None):
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    parser.add_argument("--ufos", nargs="+", help="path to ufos", required=True)
    parser.add_argument("--attribs", nargs="+", help="Attributes to set", required=True)
    args = parser.parse_args(args)

    attribs = parse_attributes(args.attribs)
    ufos = [Font(fp) for fp in args.ufos]
    for ufo in ufos:
        for attrib_name, attrib_value in attribs.items():
            set_ufo_attrib(ufo, attrib_name.split("."), attrib_value)
        ufo.save()


if __name__ == "__main__":
    main()



"""Build a Noto font from one or more source files.

By default, places unhinted TTF, hinted TTF, OTF and (if possible) variable
fonts into the ``fonts/`` directory; merges in requested subsets at the UFO
level and places *those* fonts in the ``fonts/full/`` directory.

Example:

    python3 -m gftools.actions.notobuilder src/config.yaml
"""
import logging
import os
import re
import sys
import shutil
import tempfile
from pathlib import Path

import pygit2
import ufoLib2
from fontTools import designspaceLib
from glyphsets.codepoints import CodepointsInSubset
from strictyaml import HexInt, Map, Optional, Seq, Str

from gftools.builder import GFBuilder
from gftools.builder.autohint import autohint
from gftools.builder.schema import schema
from gftools.ufomerge import merge_ufos

# Add our keys to schema
subsets_schema = Seq(
    Map(
        {
            "from": Str(),
            Optional("name"): Str(),
            Optional("ranges"): Seq(Map({"start": HexInt(), "end": HexInt()})),
        }
    )
)
_newschema = schema._validator
_newschema[Optional("includeSubsets")] = subsets_schema


class NotoBuilder(GFBuilder):
    schema = Map(_newschema)

    def __init__(self, config):
        self.config = self.load_config(config)
        if os.path.dirname(config):
            os.chdir(os.path.dirname(config))
        family_dir = self.get_family_name().replace(" ", "")
        self.config["vfDir"] = "../fonts/%s/unhinted/variable-ttf" % family_dir
        self.config["otDir"] = "../fonts/%s/unhinted/otf" % family_dir
        self.config["ttDir"] = "../fonts/%s/unhinted/ttf" % family_dir
        self.config["buildWebfont"] = False
        self.config["autohintTTF"] = False  # We take care of it ourselves
        self.outputs = set()
        self.logger = logging.getLogger("GFBuilder")
        self.fill_config_defaults()

    def get_family_name(self, source=None):
        if not source:
            source = self.config["sources"][0]
        source, _ = os.path.splitext(os.path.basename(source))
        fname = re.sub(r"([a-z])([A-Z])", r"\1 \2", source)
        fname = re.sub("-?MM$", "", fname)
        return fname

    def post_process_ttf(self, filename):
        super().post_process_ttf(filename)
        self.outputs.add(filename)
        hinted_dir = self.config["ttDir"].replace("unhinted", "hinted")
        os.makedirs(hinted_dir, exist_ok=True)
        hinted = filename.replace("unhinted", "hinted")
        try:
            autohint(filename, hinted, add_script=True)
        except Exception as e:
            self.logger.error("Couldn't autohint %s: %s" % (filename, e))
            # We just copy it and pretend.
            shutil.copy(filename, hinted)
        self.outputs.add(hinted)

    def post_process(self, filename):
        super().post_process(filename)
        self.outputs.add(filename)

    def build_variable(self):
        try:
            super().build_variable()
        except Exception as e:
            self.logger.error("Couldn't build variable font: %s" % e)

    def glyphs_to_ufo(self, source, directory=None):
        source = Path(source)
        if directory is None:
            directory = source.resolve().parent
        self.run_fontmake(
            str(source.resolve()),
            {
                "format": ["ufo"],
                "output_dir": directory,
                "master_dir": directory,
                "designspace_path": Path(directory)
                / source.with_suffix(".designspace").name,
            },
        )
        return str(Path(directory) / source.with_suffix(".designspace").name)

    def build(self):
        # First convert to Designspace/UFO
        for ix, source in enumerate(self.config["sources"]):
            if source.endswith(".glyphs"):
                self.config["sources"][ix] = self.glyphs_to_ufo(source)

        # Do a basic build first
        super().build()

        # Merge UFOs
        if not "includeSubsets" in self.config:
            return

        for key in ["vfDir", "otDir", "ttDir"]:
            self.config[key] = self.config[key].replace("unhinted", "full")

        new_builder_sources = []
        temporaries = []

        for ds_file in self.config["sources"]:
            new_ds_file_dir = tempfile.TemporaryDirectory()
            temporaries.append(new_ds_file_dir)
            ds = designspaceLib.DesignSpaceDocument.fromfile(ds_file)
            for master in ds.sources:
                # Save a copy to temporary UFO
                newpath = os.path.join(new_ds_file_dir.name, os.path.basename(master.path))
                original_ufo = ufoLib2.Font.open(master.path)
                original_ufo.save(newpath, overwrite=True)

                master.path = newpath

                for subset in self.config["includeSubsets"]:
                    self.add_subset(ds, master, subset)
            # # Set instance filenames to temporary
            for instance in ds.instances:
                instance.filename = instance.path = os.path.join(new_ds_file_dir.name, os.path.basename(instance.filename))

            # Save new designspace to temporary
            new_ds_file = os.path.join(new_ds_file_dir.name, os.path.basename(ds_file))
            ds.write(new_ds_file)

            new_builder_sources.append(new_ds_file)

        self.config["sources"] = new_builder_sources

        super().build()
        # Temporaries should get cleaned here.

    def add_subset(self, ds, ds_source, subset):
        if "name" in subset:
            # Resolve to glyphset
            unicodes = CodepointsInSubset(subset["name"])
        else:
            unicodes = []
            for r in subset["ranges"]:
                for cp in range(r["start"], r["end"] + 1):
                    unicodes.append(cp)
        location = dict(ds_source.location)
        for axis in ds.axes:
            location[axis.name] = axis.map_backward(location[axis.name])
        source_ufo = self.obtain_noto_ufo(subset["from"], location)
        target_ufo = ufoLib2.Font.open(ds_source.path)
        merge_ufos(
            target_ufo, source_ufo, codepoints=unicodes, existing_handling="skip",
        )
        target_ufo.save(ds_source.path, overwrite=True)

    def obtain_noto_ufo(self, font_name, location):
        if font_name == "Noto Sans":
            self.clone_for_subsetting("latin-greek-cyrillic")
            path = "../subset-files/latin-greek-cyrillic/sources/NotoSans-MM.glyphs"
        if font_name == "Noto Serif":
            self.clone_for_subsetting("latin-greek-cyrillic")
            path = "../subset-files/latin-greek-cyrillic/sources/NotoSerif-MM.glyphs"
        if font_name == "Noto Sans Devanagari":
            self.clone_for_subsetting("devanagari")
            path = "../subset-files/devanagari/sources/NotoSansDevanagari.glyphs"

        if path.endswith(".glyphs"):
            ds_path = path.replace(".glyphs", ".designspace")
            if os.path.exists(ds_path):
                path = ds_path
            else:
                self.logger.info("Building UFO file for subset font "+font_name)
                path = self.glyphs_to_ufo(path)
        source_ds = designspaceLib.DesignSpaceDocument.fromfile(path)

        # Find a source for this location
        return ufoLib2.Font.open(self.find_source(source_ds, location, font_name).path)

    def find_source(self, source_ds, location, font_name):
        source_mappings = {
            ax.name: ax.map_forward for ax in source_ds.axes
        }
        target = None
        for source in source_ds.sources:
            match = True
            for axis, loc in location.items():
                if axis in source.location and axis in source_mappings and source.location[axis] != source_mappings[axis](loc):
                    match = False
            if match:
                target = source
                break
        if target:
            self.logger.info(f"Adding subset from {target} for location {location}")
            return target
        self.logger.error(f"Could not find master in {font_name} for location {location}")
        raise ValueError("Could not add subset")

    def clone_for_subsetting(self, repo):
        dest = "../subset-files/" + repo
        if os.path.exists(dest):
            return
        if not os.path.exists("../subset-files"):
            os.mkdir("../subset-files")
        print(f"Cloning notofonts/{repo}")
        pygit2.clone_repository(f"https://github.com/notofonts/{repo}", dest)




if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="Build a Noto font")
    parser.add_argument("config", metavar="YAML", help="config files")
    parser.add_argument("--verbose", "-v", action="store_true", help="verbose logging")

    args = parser.parse_args()

    if args.verbose:
        logging.basicConfig(level=logging.INFO)
    builder = NotoBuilder(args.config)
    builder.build()
    print("Produced the following files:")
    for o in builder.outputs:
        print("* " + o)



import logging
import os
import subprocess

from gftools.github import GitHubClient
from gftools.utils import mkdir
try:
    from diffenator2 import ninja_diff, ninja_proof
except ModuleNotFoundError:
    raise ModuleNotFoundError(("gftools was installed without the QA "
        "dependencies. To install the dependencies, see the ReadMe, "
        "https://github.com/googlefonts/gftools#installation"))
import ninja

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class FontQA:
    def __init__(self, fonts, fonts_before=None, out="out"):
        self.fonts = fonts
        self.fonts_before = fonts_before
        self.out = out

    def diffenator(self, **kwargs):
        logger.info("Running Diffenator")
        if not self.fonts_before:
            logger.warning(
                "Cannot run Diffenator since there are no fonts before"
            )
            return
        dst = os.path.join(self.out, "Diffenator")
        ninja_diff(
            self.fonts_before,
            self.fonts,
            out=dst,
            imgs=False,
            user_wordlist=None,
            filter_styles=None,
            diffenator=True,
            diffbrowsers=False,
        )
    
    def diffbrowsers(self, imgs=False):
        logger.info("Running Diffbrowsers")
        if not self.fonts_before:
            logger.warning(
                "Cannot run diffbrowsers since there are no fonts before"
            )
            return
        dst = os.path.join(self.out, "Diffbrowsers")
        mkdir(dst)
        ninja_diff(
            self.fonts_before,
            self.fonts,
            out=dst,
            imgs=imgs,
            filter_styles=None,
            user_wordlist=None,
            diffenator=False,
            diffbrowsers=True,
        )
    
    def proof(self, imgs=False):
        logger.info("Running proofing tools")
        dst = os.path.join(self.out, "Proof")
        mkdir(dst)
        ninja_proof(
            fonts=self.fonts,
            out=dst,
            imgs=imgs,
            filter_styles=None,
        )

    def fontbakery(self, profile="googlefonts", html=False, extra_args=None):
        logger.info("Running Fontbakery")
        out = os.path.join(self.out, "Fontbakery")
        mkdir(out)
        cmd = (
            ["fontbakery", "check-"+profile, "-l", "INFO", "--succinct"]
            + [f.reader.file.name for f in self.fonts]
            + ["-C"]
            + ["--ghmarkdown", os.path.join(out, "report.md")]
        )
        if html:
            cmd.extend(["--html", os.path.join(out, "report.html")])
        if extra_args:
            cmd.extend(extra_args)
        subprocess.call(cmd)

    def googlefonts_upgrade(self, imgs=False):
        self.fontbakery()
        self.diffenator()
        self.diffbrowsers(imgs)

    def googlefonts_new(self, imgs=False):
        self.fontbakery()
        self.proof(imgs)

    def render(self, imgs=False):
        if self.fonts_before:
            self.diffbrowsers(imgs)
        else:
            self.proof(imgs)

    def post_to_github(self, url):
        """Post Fontbakery report as a new issue or as a comment to an open
        PR"""
        # Parse url tokens
        url_split = url.split("/")
        repo_owner = url_split[3]
        repo_name = url_split[4]
        issue_number = url_split[-1] if "pull" in url else None

        fontbakery_report = os.path.join(self.out, "Fontbakery", "report.md")
        if not os.path.isfile(fontbakery_report):
            logger.warning(
                "Cannot Post Github message because no Fontbakery report exists"
            )
            return
        
        client = GitHubClient(repo_owner, repo_name)

        with open(fontbakery_report) as doc:
            msg = doc.read()
            if issue_number:
                client.create_issue_comment(issue_number, msg)
            else:
                client.create_issue("Google Font QA report", msg)



#!/usr/bin/env python3
#
# Copyright 2014 Google Inc. All rights reserved.
# Copyright 2021 The Google Font Tools Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Drop hints from a font."""
import argparse
import sys
from fontTools.ttLib import TTFont

parser = argparse.ArgumentParser(description=__doc__)
parser.add_argument("input", help="font file to process")
parser.add_argument("output", help="file to save font", nargs="?")


def drop_hints_from_glyphs(font):
    """Drops the hints from a font's glyphs."""
    glyf_table = font["glyf"]
    for glyph_index in range(len(glyf_table.glyphOrder)):
        glyph_name = glyf_table.glyphOrder[glyph_index]
        glyph = glyf_table[glyph_name]
        if glyph.numberOfContours > 0:
            if glyph.program.bytecode:
                glyph.program.bytecode = array.array("B")


def drop_tables(font, tables):
    """Drops the listed tables from a font."""
    for table in tables:
        if table in font:
            del font[table]


def main(args=None):
    """Drop the hints from the first file specified and save as second."""
    args = parser.parse_args(args)
    font = TTFont(args.input)

    drop_hints_from_glyphs(font)
    drop_tables(font, ["cvt ", "fpgm", "hdmx", "LTSH", "prep", "VDMX"])

    if not args.output:
        args.output = args.input

    font.save(args.output)


if __name__ == "__main__":
    main()



"""
This schema represents all known key/value pairs for the builder config file.
"""
from strictyaml import (
                        Map,
                        MapPattern,
                        Str,
                        Int,
                        Float,
                        Seq,
                        Optional,
                        Bool,
                        UniqueSeq,
                        Enum
                        )
from gftools.packager import CATEGORIES


stat_schema = Seq(
    Map({
        "name": Str(),
        "tag": Str(),
        "values": Seq(
            Map({
                "name": Str(),
                "value": Int() | Float(),
                Optional("nominalValue"): Int() | Float(),
                Optional("linkedValue"): Int() | Float(),
                Optional("rangeMinValue"): Int() | Float(),
                Optional("rangeMaxValue"): Int() | Float(),
                Optional("flags"): Int()
            })
        )
    }),
)

stat_format4_schema = Seq(
    Map({
        "name": Str(),
        Optional("flags"): Int(),
        "location": MapPattern(Str(), Int() | Float()),
    })
)

instance_schema = MapPattern(Str(), Seq(
    Map({
        Optional("familyName"): Str(),
        Optional("styleName"): Str(),
        "coordinates": MapPattern(Str(), Int() | Float()),
    })
))

schema = Map(
    {
        "sources": Seq(Str()),
        Optional("vttSources"): MapPattern(Str(), Str()),
        Optional("fvarInstanceAxisDflts"): MapPattern(Str(), Float()),
        Optional("logLevel"): Str(),
        Optional("stylespaceFile"): Str(),
        Optional("stat"): stat_schema | MapPattern(Str(), stat_schema),
        Optional("statFormat4"): stat_format4_schema | MapPattern(Str(), stat_format4_schema),
        Optional("familyName"): Str(),
        Optional("includeSourceFixes"): Bool(),
        Optional("stylespaceFile"): Str(),
        Optional("instances"): instance_schema,
        Optional("buildVariable"): Bool(),
        Optional("buildStatic"): Bool(),
        Optional("buildOTF"): Bool(),
        Optional("buildTTF"): Bool(),
        Optional("buildWebfont"): Bool(),
        Optional("outputDir"): Str(),
        Optional("vfDir"): Str(),
        Optional("ttDir"): Str(),
        Optional("otDir"): Str(),
        Optional("woffDir"): Str(),
        Optional("cleanUp"): Bool(),
        Optional("autohintTTF"): Bool(),
        Optional("axisOrder"): Seq(Str()),
        Optional("flattenComponents"): Bool(),
        Optional("decomposeTransformedComponents"): Bool(),
        Optional("ttfaUseScript"): Bool(),
        Optional("googleFonts"): Bool(),
        Optional("category"): UniqueSeq(Enum(CATEGORIES)),
        Optional("reverseOutlineDirection"): Bool(),
        Optional("interpolate"): Bool(),
        Optional("removeOutlineOverlaps"): Bool(),
        Optional("expandFeaturesToInstances"): Bool(),
        Optional("version"): Str(),
        Optional("addGftoolsVersion"): Bool(),
    }
)



from gftools.instancer import gen_static_font
from fontTools.ttLib import TTFont
import pytest
import os

TEST_DATA = os.path.join("data", "test")


@pytest.fixture
def var_font():
    """VF family consisting of a single font with two axes, wdth, wght"""
    return TTFont(os.path.join(TEST_DATA, "Inconsolata[wdth,wght].ttf"))


def _name_record(ttFont, nameID):
    nametable = ttFont['name']
    record = nametable.getName(nameID, 3, 1, 0x409)
    if record:
        return record.toUnicode()
    return None


def test_gen_static_font(var_font):
    static_font = gen_static_font(var_font, {"wght": 600, "wdth": 75})
    assert _name_record(static_font, 1) == "Inconsolata Condensed SemiBold"
    assert _name_record(static_font, 2) == "Regular"
    assert _name_record(static_font, 16) == "Inconsolata Condensed"
    assert _name_record(static_font, 17) == "SemiBold"

    assert static_font['OS/2'].usWeightClass == 600
    assert static_font['OS/2'].usWidthClass == 5
    assert static_font['OS/2'].fsSelection & (1 << 6)
    assert static_font['head'].macStyle == 0


def test_gen_static_font_custom_names(var_font):
    static_font = gen_static_font(var_font, {"wght": 900}, "Custom Family", "Black")
    assert _name_record(static_font, 1) == "Custom Family Black"
    assert _name_record(static_font, 2) == "Regular"
    assert _name_record(static_font, 16) == "Custom Family"
    assert _name_record(static_font, 17) == "Black"


def test_gen_static_font_custom_names_without_declaring_wght(var_font):
    static_font = gen_static_font(var_font, {"wght": 900}, "Custom Family", "8pt SemiCondensed")
    assert _name_record(static_font, 1) == "Custom Family 8pt SemiCondensed"
    assert _name_record(static_font, 2) == "Regular"
    assert _name_record(static_font, 16) == None
    assert _name_record(static_font, 17) == None


def test_gen_static_font_custom_names_ribbi(var_font):
    static_font = gen_static_font(var_font, {"wght": 900}, "Custom Family", "8pt SemiCondensed Bold Italic")
    assert _name_record(static_font, 1) == "Custom Family 8pt SemiCondensed"
    assert _name_record(static_font, 2) == "Bold Italic"
    assert _name_record(static_font, 16) == None
    assert _name_record(static_font, 17) == None


def test_gen_static_font_custom_names_non_ribbi(var_font):
    static_font = gen_static_font(var_font, {"wght": 900}, "Custom Family", "8pt SemiCondensed Medium")
    assert _name_record(static_font, 1) == "Custom Family 8pt SemiCondensed Medium"
    assert _name_record(static_font, 2) == "Regular"
    assert _name_record(static_font, 16) == "Custom Family 8pt SemiCondensed"
    assert _name_record(static_font, 17) == "Medium"


import argparse
import subprocess
import os
from github import Github
import re

def get_latest_release(family, user=None, repo=None):
    if not (user and repo):
        repo_url = subprocess.check_output(["git", "remote", "get-url", "origin"]).decode("utf8").strip()
        url_split = repo_url.split("/")
        user, repo = url_split[3], url_split[4]

    g = Github(os.environ["GITHUB_TOKEN"])
    repo = g.get_repo(user + '/' + repo)
    for release in repo.get_releases():
        if release.draft:
            continue
        m = re.match(r"^(.*)-(v[\d.]+)", release.tag_name)
        if not m:
            print(f"Unparsable release {release.tag_name} in {repo_name}")
            continue
        thisfamily, version = m[1], m[2]
        if thisfamily != family:
            continue
        assets = release.get_assets()
        download_url = assets[0].browser_download_url
        return version, download_url
    return None, None

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Return the URL of a font's latest release artefact")
    parser.add_argument('--user', help='the repository username', default="notofonts")
    parser.add_argument('--repo', help='the repository name')
    parser.add_argument('family', help='the font family name')
    args = parser.parse_args()

    version, download_url = get_latest_release(args.family, args.user, args.repo)
    if version and download_url:
        print(f"::set-output name=version::{version}")
        print(f"::set-output name=url::{download_url}")




#MenuTitle: write simple GlyphData
# -*- coding: utf-8 -*-

infos = GSGlyphsInfo.sharedManager().glyphInfos()
# or:
# infos = GSGlyphsInfo.alloc().initWithLocalFile_(NSURL.fileURLWithPath_("path to custom GlyphData.xml file"))

f = open("GlyphData.xml", "w")
fIdeo = open("GlyphData_Ideographs.xml", "w")

def writeHeader(f):
	f.write('<?xml version="1.0" encoding="UTF-8" ?>\n\
<!DOCTYPE glyphData [\n\
<!ELEMENT glyphData (glyph)+>\n\
<!ELEMENT glyph EMPTY>\n\
<!ATTLIST glyph\n\
	unicode			CDATA		#IMPLIED\n\
	name			CDATA		#REQUIRED\n\
	category		CDATA		#REQUIRED\n\
	subCategory		CDATA		#IMPLIED\n\
	script			CDATA		#IMPLIED\n\
	description		CDATA		#REQUIRED\n\
	production		CDATA		#IMPLIED\n\
	altNames		CDATA		#IMPLIED>\n\
]>\n\
<glyphData>\n')

writeHeader(f)
writeHeader(fIdeo)

disabledGlyphs = ["brevecomb_acutecomb",
				"brevecomb_gravecomb",
				"brevecomb_hookabovecomb",
				"brevecomb_tildecomb",
				"circumflexcomb_acutecomb",
				"circumflexcomb_gravecomb",
				"circumflexcomb_hookabovecomb", 
				"circumflexcomb_tildecomb",
				"idotaccent.sc", 
				"i.sc",
				]
forcedGlyphs = ["ringcenter-ar"]

def printInfo(info):
	string = '	<glyph '
	if info.unicode:
		string += 'unicode="' + info.unicode + '" '
	string += 'name="' + info.name + '" '
# 	if info.sortName:
# 		string += 'sortName="' + info.sortName + '" '
	if info.category:
		string += 'category="' + info.category + '" '
	if info.subCategory and info.subCategory != "Other":
		string += 'subCategory="' + info.subCategory + '" '
	if info.script:
		string += 'script="' + info.script + '" '
	if info.productionName:
		string += 'production="' + info.productionName + '" '
	if info.altNames:
		string += 'altNames="' + ", ".join(info.altNames) + '" '
	if info.desc:
		string += 'description="' + info.desc + '" '
	
	string += '/>\n'
	return string
	
count = 0
for info in infos:
	if info.script == "han":
		fIdeo.write(printInfo(info))
	elif (info.name in disabledGlyphs or (info.script == "arabic" and info.unicode is None)) and (info.name not in forcedGlyphs):
		continue
	else:
		f.write(printInfo(info))
	count += 1

print "Written %d entries" % count
f.write('</glyphData>\n')
fIdeo.write('</glyphData>\n')

f.close()
fIdeo.close()


#!/usr/bin/env python3
# Copyright 2017 The Font Bakery Authors.
# Copyright 2017 The Google Font Tools Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import argparse
import os
import tabulate
from fontTools import ttLib
from gftools.utils import has_mac_names
from gftools.fix import drop_mac_names, drop_superfluous_mac_names, FontFixer


parser = argparse.ArgumentParser(description='Print out nameID'
                                             ' strings of the fonts')
parser.add_argument('font', nargs="+")
parser.add_argument('--autofix', default=False,
                    action='store_true', help='Apply autofix')
parser.add_argument('--csv', default=False, action='store_true',
                    help="Output data in comma-separate-values"
                         " (CSV) file format")
parser.add_argument('--id', '-i', default='all')
parser.add_argument('--platform', '-p', type=int, default=3)
parser.add_argument('--drop-superfluous-mac-names', '-ms', default=False,
                    action='store_true',
                    help='Drop superfluous Mac names')
parser.add_argument('--drop-mac-names', '-m', default=False,
                    action='store_true',
                    help='Drop all Mac name fields')


def delete_non_platform1_names(font):
    changed = False
    for name in font['name'].names:
        if name.platformID != 1:
            del name
            changed = True
    return changed

def main(args=None):
    args = parser.parse_args(args)
    nameids = ['1', '2', '4', '6', '16', '17', '18']
    user_nameids = [x.strip() for x in args.id.split(',')]

    if 'all' not in user_nameids:
        nameids = set(nameids) & set(user_nameids)

    rows = []
    for font in args.font:
        ttfont = ttLib.TTFont(font)
        row = [os.path.basename(font)]
        for name in ttfont['name'].names:
            if str(name.nameID) not in nameids or\
               name.platformID != args.platform:
                continue

            value = name.string.decode(name.getEncoding()) or ''
            row.append(value)

        rows.append(row)

    header = ['filename'] + ['id' + x for x in nameids]

    def as_csv(rows):
        import csv
        import sys
        writer = csv.writer(sys.stdout)
        writer.writerows([header])
        writer.writerows(rows)
        sys.exit(0)

    if args.csv:
        as_csv(rows)

    print(tabulate.tabulate(rows, header, tablefmt="pipe"))

    for path in args.font:
        fixer = FontFixer(path, verbose=True)
        if args.autofix:
            fixer.fixes.append(delete_non_platform1_names)
        if args.drop_superfluous_mac_names:
            if has_mac_names(ttLib.TTFont(path)):
                fixer.fixes.append(drop_superfluous_mac_names)
            else:
                print('font %s has no mac nametable' % path)

        if args.drop_mac_names:
            if has_mac_names(ttLib.TTFont(path)):
                fixer.fixes.append(drop_mac_names)
            else:
                print('font %s has no mac nametable' % path)

        fixer.fix()


if __name__ == '__main__':
    main()



"""
gftools-builder: Config-driven font project builder
===================================================

This utility wraps fontmake and a number of post-processing fixes to
build variable, static OTF, static TTF and webfonts from Glyphs,
Designspace/UFO or UFO sources.

It should be instantiated with a configuration file, typically ``config.yaml``,
which looks like this::

    sources:
      - Texturina.glyphs
      - Texturina-Italic.glyphs
    axisOrder:
      - opsz
      - wght
    outputDir: "../fonts"
    familyName: Texturina
    version: 1.005
    stat:
      - name: Width
        tag: wdth
        values:
        - name: UltraCondensed
          value: 50
          ...
      - name: Weight
        tag: wght
        values:
        - name: Hairline
          rangeMinValue: 1
          nominalValue: 1
          rangeMaxValue: 50
        ...
    statFormat4:
      - name: Green
        location:
          wght: 300
          wdth: 200
      - name: Blue
        location:
          wght: 400
          wdth: 200
    ...
    instances:
      Texturina[wght].ttf:
      - coordinates:
          wght: 400
      - coordinates:
          wght: 500
      - familyName: "Texturina Exotic"
        styleName: "Medium"
        coordinates:
          wght: 500
        ...
      Texturina-Italic[wght].ttf:
      - coordinates:
          wght: 700
        ...
    vttSources:
      Texturina[wght].ttf: vtt-roman.ttx
      Texturina-Italic[wght].ttf: vtt-italic.ttx
    ...

To build a font family from the command line, use:

    gftools builder path/to/config.yaml

The config file may contain the following keys. The ``sources`` key is
required, all others have sensible defaults:

* ``sources``: Required. An array of Glyphs, UFO or designspace source files.
* ``logLevel``: Debugging log level. Defaults to ``INFO``.
* ``stylespaceFile``: A statmake ``.stylespace`` file.
* ``stat``: A STAT table configuration. This may be either a list of axes and
  values as demonstrated above, or a dictionary mapping each variable font to a
  per-source list. If neither ``stylespaceFile`` or ``stat`` are provided, a
  STAT table is generated automatically using ``gftools.stat``.
* ``instances``: A list of static font TTF instances to generate from each variable
  font as demonstrated above. If this argument isn't provided, static TTFs will
  be generated for each instance that is specified in the source files.
* ``buildVariable``: Build variable fonts. Defaults to true.
* ``buildStatic``: Build static fonts (OTF or TTF depending on ``$buildOTF``
  and ``$buildTTF`). Defaults to true.
* ``buildOTF``: Build OTF fonts. Defaults to true.
* ``buildTTF``: Build TTF fonts. Defaults to true.
* ``buildWebfont``: Build WOFF2 fonts. Defaults to ``$buildStatic``.
* ``outputDir``: Where to put the fonts. Defaults to ``../fonts/``
* ``vfDir``: Where to put variable fonts. Defaults to ``$outputDir/variable``.
* ``ttDir``: Where to put TrueType static fonts. Defaults to ``$outputDir/ttf``.
* ``otDir``: Where to put CFF static fonts. Defaults to ``$outputDir/otf``.
* ``woffDir``: Where to put WOFF2 static fonts. Defaults to ``$outputDir/webfonts``.
* ``cleanUp``: Whether or not to remove temporary files. Defaults to ``true``.
* ``autohintTTF``: Whether or not to autohint TTF files. Defaults to ``true``.
* ``ttfaUseScript``: Whether or not to detect a font's primary script and add a ``-D<script>`` flag to ttfautohint. Defaults to ``false``.
* ``vttSources``: To patch a manual VTT hinting program (ttx format) to font binaries.
* ``axisOrder``: STAT table axis order. Defaults to fvar order.
* ``familyName``: Family name for variable fonts. Defaults to family name of first source file.
* ``flattenComponents``: Whether to flatten components on export. Defaults to ``true``.
* ``decomposeTransformedComponents``: Whether to decompose transformed components on export. Defaults to ``true``.
* ``googleFonts``: Whether this font is destined for release on Google Fonts. Used by GitHub Actions. Defaults to ``false``.
* ``category``: If this font is destined for release on Google Fonts, a list of the categories it should be catalogued under. Used by GitHub Actions. Must be set if ``googleFonts`` is set.
* ``fvarInstanceAxisDflts``: Mapping to set every fvar instance's non-wght axis
  value e.g if a font has a wdth and wght axis, we can set the wdth to be 100 for
  every fvar instance. Defaults to ``None``
* ``expandFeaturesToInstances``: Resolve all includes in the sources' features, so that generated instances can be compiled without errors. Defaults to ``true``.
* ``reverseOutlineDirection``: Reverse the outline direction when compiling TTFs (no effect for OTFs). Defaults to fontmake's default.
* ``removeOutlineOverlaps``: Remove overlaps when compiling fonts. Defaults to fontmake's default.
* ``interpolate``: Enable fontmake --interpolate flag. Defaults to ``false``.
"""

from fontmake.font_project import FontProject
from fontTools import designspaceLib
from fontTools.otlLib.builder import buildStatTable
from fontTools.ttLib import TTFont, newTable
from fontTools.ttLib.woff2 import main as woff2_main
from fontv.libfv import FontVersion
from gftools.builder.schema import schema
from gftools.builder.autohint import autohint
from gftools.fix import fix_font
from gftools.stat import gen_stat_tables, gen_stat_tables_from_config
from gftools.utils import font_is_italic, font_familyname, font_stylename
from gftools.instancer import gen_static_font
from strictyaml import load, YAMLError
from strictyaml.exceptions import YAMLValidationError
from ufo2ft import CFFOptimization
from ufo2ft.filters.flattenComponents import FlattenComponentsFilter
from ufo2ft.filters.decomposeTransformedComponents import DecomposeTransformedComponentsFilter
from vttLib.transfer import merge_from_file as merge_vtt_hinting
from vttLib import compile_instructions as compile_vtt_hinting
import difflib
import gftools
import glyphsLib
from defcon import Font
import argparse
import logging
import os
import re
import shutil
import statmake.classes
import statmake.lib
import sys
import tempfile


class GFBuilder:
    schema = schema

    def __init__(self, configfile=None, config=None):
        if configfile:
            self.config = self.load_config(configfile)
            if os.path.dirname(configfile):
                os.chdir(os.path.dirname(configfile))
        else:
            self.config = config
        self.logger = logging.getLogger("GFBuilder")
        self.outputs = set()  # A list of files we created
        self.fill_config_defaults()

    def load_config(self, configfile):
        with open(configfile) as f:
            unprocessed_yaml = f.read()
        try:
            return load(unprocessed_yaml, self.schema).data
        except YAMLValidationError as e:
            if "unexpected key not in schema" in e.problem:
                bad_key = str(e.problem)
                raise YAMLError(
                    f"\nA key in the configuration file, typically ``config.yaml``, is likely misspelled."
                    f"\nError caused by: {bad_key}"
                )
            else:
                raise ValueError(
                    "The yaml config file isn't structured properly. Please refer to: "
                    "https://github.com/googlefonts/gftools/blob/main/Lib/gftools/builder/__init__.py#L7"
                )

    def build(self):
        loglevel = getattr(logging, self.config["logLevel"].upper())
        logging.basicConfig(level=loglevel)
        self.logger.setLevel(loglevel)
        # Shut up irrelevant loggers
        if loglevel != logging.DEBUG:
            for irrelevant in [
                "glyphsLib.classes",
                "glyphsLib.builder.components",
                "ufo2ft",
                "ufo2ft.filters",
                "ufo2ft.postProcessor",
                "fontTools.varLib",
                "fontTools.feaLib.parser",
                "cu2qu.ufo",
                "glyphsLib.builder.builders.UFOBuilder",
            ]:
                logging.getLogger(irrelevant).setLevel(logging.ERROR)

        # Store the current files/dirs listed in the source dir. Using
        # self.config['cleanUp'] will delete the auxiliary dirs/files which
        # are created whilst building so we need know the original
        # dir state in order to do this.
        source_files = os.listdir()

        if self.config["buildVariable"]:
            self.build_variable()
            # transfer vf vtt hints now in case static fonts are instantiated
            if "vttSources" in self.config:
                self.build_vtt(self.config['vfDir'])
        if self.config["buildStatic"]:
            self.build_static()
            if "vttSources" in self.config:
                self.build_vtt(self.config['ttDir'])
        # All done
        self.logger.info(
            "Building %s completed. All done!" % (
                ", ".join([
                    b.lstrip("build").lower()
                    for b in ["buildVariable", "buildStatic", "buildWebfont"]
                    if self.config[b]
                ])
            )
        )
        if self.config["cleanUp"]:
            files_added_during_build = set(os.listdir()) - set(source_files)
            if not files_added_during_build:
                return
            self.logger.info(
                f"Removing auxiliary dirs/files {files_added_during_build}"
            )
            for file_ in files_added_during_build:
                self.rm(file_)

    def get_family_name(self):
        """Ensure that all source files have the same family name"""
        names = set()
        for fp in self.config["sources"]:
            if fp.endswith("glyphs"):
                src = glyphsLib.GSFont(fp)
                names.add(src.familyName)
            elif fp.endswith("ufo"):
                src = Font(fp)
                names.add(src.info.familyName)
            elif fp.endswith("designspace"):
                ds = designspaceLib.DesignSpaceDocument.fromfile(self.config["sources"][0])
                names.add(ds.sources[0].familyName)
            else:
                raise ValueError(f"{fp} not a supported source file!")

        if len(names) > 1:
            raise ValueError(
                f"Inconsistent family names in sources {names}. Set familyName in config instead"
            )
        return list(names)[0]

    def determine_variability(self):
        is_variable = []
        for fp in self.config["sources"]:
            if fp.endswith("glyphs"):
                src = glyphsLib.GSFont(fp)
                is_variable.append(len(src.masters) > 1)
            elif fp.endswith("ufo"):
                is_variable.append(False)
            elif fp.endswith("designspace"):
                ds = designspaceLib.DesignSpaceDocument.fromfile(self.config["sources"][0])
                is_variable.append(len(ds.sources) > 1)
        # This needs to be consistent...
        if any(variability != is_variable[0] for variability in is_variable[1:]):
            raise ValueError("Some sources were multi-master and some were not. Specify buildVariable manually.")
        return is_variable[0]

    def fill_config_defaults(self):
        if "familyName" not in self.config:
            self.logger.info("Deriving family name (this takes a while)")
            self.config["familyName"] = self.get_family_name()
        if "outputDir" not in self.config:
            self.config["outputDir"] = "../fonts"
        if "vfDir" not in self.config:
            self.config["vfDir"] = self.config["outputDir"] + "/variable"
        if "ttDir" not in self.config:
            self.config["ttDir"] = self.config["outputDir"] + "/ttf"
        if "otDir" not in self.config:
            self.config["otDir"] = self.config["outputDir"] + "/otf"
        if "woffDir" not in self.config:
            self.config["woffDir"] = self.config["outputDir"] + "/webfonts"

        if "buildVariable" not in self.config:
            self.config["buildVariable"] = self.determine_variability()
        if "buildStatic" not in self.config:
            self.config["buildStatic"] = True
        if "buildOTF" not in self.config:
            self.config["buildOTF"] = True
        if "buildTTF" not in self.config:
            self.config["buildTTF"] = True
        if "buildWebfont" not in self.config:
            self.config["buildWebfont"] = self.config["buildStatic"]
        if "autohintTTF" not in self.config:
            self.config["autohintTTF"] = True
        if "ttfaUseScript" not in self.config:
            self.config["ttfaUseScript"] = False
        if "logLevel" not in self.config:
            self.config["logLevel"] = "INFO"
        if "cleanUp" not in self.config:
            self.config["cleanUp"] = True
        if "includeSourceFixes" not in self.config:
            self.config["includeSourceFixes"] = False
        if "fvarInstanceAxisDflts" not in self.config:
            self.config["fvarInstanceAxisDflts"] = None
        if "flattenComponents" not in self.config:
            self.config["flattenComponents"] = True
        if "addGftoolsVersion" not in self.config:
            self.config["addGftoolsVersion"] = True
        if "decomposeTransformedComponents" not in self.config:
            self.config["decomposeTransformedComponents"] = True
        if "interpolate" not in self.config:
            self.config["interpolate"] = False

    def build_variable(self):
        self.mkdir(self.config["vfDir"], clean=True)
        ttFonts = []
        for source in self.config["sources"]:
            args = {"output": ["variable"], "family_name": self.config["familyName"]}
            if not source.endswith(".designspace") and not source.endswith("glyphs"):
                continue
            self.logger.info("Creating variable fonts from %s" % source)
            sourcebase = os.path.splitext(os.path.basename(source))[0]
            args["output_path"] = os.path.join(
                self.config["vfDir"], sourcebase + "-VF.ttf",
            )
            output_files = self.run_fontmake(source, args)
            newname = self.rename_variable(output_files[0])
            ttFont = TTFont(newname)
            ttFonts.append(ttFont)

        if not ttFonts:
            return

        self.gen_stat(ttFonts)
        # We post process each variable font after generating the STAT tables
        # because these tables are needed in order to fix the name tables.
        for ttFont in ttFonts:
            self.post_process_variable(ttFont.reader.file.name)
            self.outputs.add(ttFont.reader.file.name)

    def run_fontmake(self, source, args):
        if "output_dir" in args:
            original_output_dir = args["output_dir"]
            tmpdir = tempfile.TemporaryDirectory()
            args["output_dir"] = tmpdir.name

        filters = args.get("filters", [])
        if (
            self.config["flattenComponents"] or
            self.config["decomposeTransformedComponents"]
        ):
            if self.config["flattenComponents"]:
                filters.append(
                    FlattenComponentsFilter()
                )

            if self.config["decomposeTransformedComponents"]:
                filters.append(
                    DecomposeTransformedComponentsFilter()
                )
        # ... will run the filters in the ufo's lib,
        # https://github.com/googlefonts/fontmake/issues/872
        args["filters"] = [...] + filters

        # The following arguments must be determined dynamically.
        if source.endswith((".glyphs", ".designspace", ".glyphspackage")):
            args["expand_features_to_instances"] = self.config.get(
                "expandFeaturesToInstances", True
            )
        # XXX: This will blow up if output formats are mixing TTFs/OTFs.
        is_ttf = args["output"][0] in {"ttf", "ttf-interpolatable", "variable"}
        if "reverseOutlineDirection" in self.config and is_ttf:
            args["reverse_direction"] = self.config["reverseOutlineDirection"]
        if "removeOutlineOverlaps" in self.config:
            args["remove_overlaps"] = self.config["removeOutlineOverlaps"]

        if source.endswith(".glyphs") or source.endswith(".glyphspackage"):
            FontProject().run_from_glyphs(source, **args)
        elif source.endswith(".designspace"):
            FontProject().run_from_designspace(source, **args)
        elif source.endswith(".ufo"):
            FontProject().run_from_ufos([source], **args)
        else:
            raise ValueError("Can't build from unknown source file: %s" % source)
        if "output_path" in args:
            return [args["output_path"]]
        else:
            # Move it to where it should be...
            file_names = os.listdir(args["output_dir"])
            for file_name in file_names:
                shutil.move(
                    os.path.join(args["output_dir"], file_name), original_output_dir
                )
            tmpdir.cleanup()
            args["output_dir"] = original_output_dir
            return [os.path.join(original_output_dir, x) for x in file_names]

    def rename_variable(self, fontfile):
        font = TTFont(fontfile)
        assert "fvar" in font
        axis_tags = sorted([ax.axisTag for ax in font["fvar"].axes])
        axis_tags = ",".join(axis_tags)
        newname = fontfile.replace("-VF.ttf", "[%s].ttf" % axis_tags)
        os.rename(fontfile, newname)
        return newname

    def gen_stat(self, varfonts):
        if "axisOrder" not in self.config:
            self.config["axisOrder"] = [ax.axisTag for ax in varfonts[0]["fvar"].axes]
        if len(varfonts) > 1 and "ital" not in self.config["axisOrder"]:
            # *Are* any italic? Presumably, but test
            if any(font_is_italic(f) for f in varfonts):
                self.config["axisOrder"].append("ital")

        locations = self.config.get("statFormat4", None)
        if locations and 'stat' not in self.config:
            raise ValueError(
                "Cannot add statFormat 4 axisValues since no stat table has been declared."
            )
        if "stylespaceFile" in self.config and self.config["stylespaceFile"]:
            self.gen_stat_stylespace(self.config["stylespaceFile"], varfonts)
        elif "stat" in self.config:
            gen_stat_tables_from_config(self.config["stat"], varfonts, locations=locations)
        else:
            gen_stat_tables(varfonts)

        for ttFont in varfonts:
            ttFont.save(ttFont.reader.file.name)

    def gen_stat_stylespace(self, stylespaceFile, varfonts):
        import warnings
        warnings.warn(".stylespace files are supported for compatibility but"
            "you are encouraged to specify your STAT table axes in the config file")
        stylespace = statmake.classes.Stylespace.from_file(stylespaceFile)
        for ttFont in varfonts:
            if "ital" in self.config["axisOrder"]:
                if font_is_italic(ttFont):
                    additional_locations = {"Italic": 1}
                else:
                    additional_locations = {"Italic": 0}
            else:
                additional_locations = {}
            statmake.lib.apply_stylespace_to_variable_font(
                stylespace, ttFont, additional_locations
            )

    def build_static(self):
        if self.config["buildOTF"]:
            self.build_a_static_format("otf", self.config["otDir"], self.post_process)
        if self.config["buildWebfont"]:
            self.mkdir(self.config["woffDir"], clean=True)
        if self.config["buildTTF"]:
            if "instances" in self.config:
                self.instantiate_static_fonts(
                    self.config["ttDir"], self.post_process_static_ttf
                )
            else:
                self.build_a_static_format(
                    "ttf", self.config["ttDir"], self.post_process_static_ttf
                )

    def instantiate_static_fonts(self, directory, postprocessor):
        self.mkdir(directory, clean=True)
        for font in self.config["instances"]:
            varfont_path = os.path.join(self.config['vfDir'], font)
            varfont = TTFont(varfont_path)
            for font, instances in self.config["instances"].items():
                for inst in instances:
                    if 'familyName' in inst:
                        family_name = inst['familyName']
                    else:
                        family_name = self.config['familyName']
                    if "styleName" in inst:
                        style_name = inst['styleName']
                    else:
                        style_name = None

                    static_font = gen_static_font(
                        varfont,
                        axes=inst["coordinates"],
                        family_name=family_name,
                        style_name=style_name,
                    )
                    family_name = font_familyname(static_font)
                    style_name = font_stylename(static_font)
                    dst = os.path.join(
                        directory, f"{family_name}-{style_name}.ttf".replace(" ", "")
                    )
                    static_font.save(dst)
                    postprocessor(dst)
                    self.outputs.add(dst)

    def build_a_static_format(self, format, directory, postprocessor):
        self.mkdir(directory, clean=True)
        for source in self.config["sources"]:
            args = {
                "output": [format],
                "output_dir": directory,
                "optimize_cff": CFFOptimization.SUBROUTINIZE,
            }
            if self.config["buildVariable"] or self.config["interpolate"]:
                args["interpolate"] = True
            self.logger.info("Creating static fonts from %s" % source)
            for fontfile in self.run_fontmake(source, args):
                self.logger.info("Created static font %s" % fontfile)
                postprocessor(fontfile)
                self.outputs.add(fontfile)

    def rm(self, fp):
        if os.path.isdir(fp):
            shutil.rmtree(fp, ignore_errors=True)
        elif os.path.isfile(fp):
            os.remove(fp)

    def mkdir(self, directory, clean=False):
        if clean:
            self.rm(directory)
        os.makedirs(directory)

    def post_process(self, filename):
        self.logger.info("Postprocessing font %s" % filename)
        self.set_version(filename)
        font = TTFont(filename)
        font = fix_font(
            font,
            include_source_fixes=self.config["includeSourceFixes"],
            fvar_instance_axis_dflts=self.config["fvarInstanceAxisDflts"]
        )
        font.save(filename)

    def post_process_ttf(self, filename):
        self.logger.debug("Deprecated method .post_process_ttf called, update code to use .post_process_static_ttf")
        self.post_process_static_ttf(filename)

    def post_process_static_ttf(self, filename):
        if self.config["autohintTTF"]:
            self.logger.debug("Autohinting")
            autohint(filename, filename, add_script=self.config["ttfaUseScript"])
        self.post_process(filename)
        self.build_webfont(filename)

    def post_process_variable(self, filename):
        self.post_process(filename)
        self.build_webfont(filename)

    def build_webfont(self, filename):
        if self.config["buildWebfont"]:
            self.logger.debug("Building webfont")
            woff2_main(["compress", filename])
            self.move_webfont(filename)

    def set_version(self, filename):
        if "version" not in self.config and not self.config["addGftoolsVersion"]:
            return
        fv = FontVersion(filename)
        if "version" in self.config:
            fv.set_version_number(self.config["version"])
        if self.config["addGftoolsVersion"]:
            fv.version_string_parts.append(f"gftools[{gftools.__version__}]")
        fv.write_version_string()


    def build_vtt(self, font_dir):
        for font, vtt_source in self.config['vttSources'].items():
            if font not in os.listdir(font_dir):
                continue
            self.logger.debug(f"Compiling hint file {vtt_source} into {font}")
            font_path = os.path.join(font_dir, font)
            font = TTFont(font_path)
            merge_vtt_hinting(font, vtt_source, keep_cvar=True)
            compile_vtt_hinting(font, ship=True)

            # Add a gasp table which is optimised for VTT hinting
            # https://googlefonts.github.io/how-to-hint-variable-fonts/
            gasp_tbl = newTable("gasp")
            gasp_tbl.gaspRange = {8: 10, 65535: 15}
            gasp_tbl.version = 1
            font['gasp'] = gasp_tbl
            font.save(font.reader.file.name)

    def move_webfont(self, filename):
        wf_filename = filename.replace(".ttf", ".woff2")
        os.rename(
            wf_filename,
            wf_filename.replace(self.config["ttDir"], self.config["woffDir"]),
        )


def main(args=None):
    parser = argparse.ArgumentParser(
        description=("Build a font family"),
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="#"*79 + "\n" + __doc__,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
        default=False,
        help="Show extra debugging information",
    )
    parser.add_argument("--family-name", help="Font family name")
    parser.add_argument(
        "--no-autohint",
        action="store_true",
        default=False,
        help="Don't run ttfautohint on static TTFs",
    )
    parser.add_argument("--stylespace", help="Path to a statmake stylespace file")

    parser.add_argument(
        "--no-clean-up",
        action="store_true",
        default=False,
        help="Do not remove temporary files (instance_ufos/)")

    parser.add_argument("file", nargs="+", help="YAML build config file *or* source files")

    parser.add_argument("--dump-config", type=str, help="Config file to generate")

    args = parser.parse_args(args)

    if len(args.file) == 1 and (
        args.file[0].endswith(".yaml") or args.file[0].endswith(".yml")
    ):
        builder = GFBuilder(configfile=args.file[0])
    else:
        config={"sources": args.file}
        if args.stylespace:
            config["stylespaceFile"] = args.stylespace
        if args.family_name:
            config["familyName"] = args.family_name
        builder = GFBuilder(config=config)

    if args.no_autohint:
        builder.config["autohintTTF"] = False

    if args.no_clean_up:
        builder.config["cleanUp"] = False

    if args.debug:
        builder.config["logLevel"] = "DEBUG"

    if args.dump_config:
        import sys
        import yaml

        with open(args.dump_config, "w") as fp:
            config= {k: v for (k, v) in builder.config.items() if v is not None}
            fp.write(yaml.dump(config, Dumper=yaml.SafeDumper))
        sys.exit()

    builder.build()

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Copyright 2016 The Fontbakery Authors
# Copyright 2017 The Google Fonts Tools Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from argparse import ArgumentParser
from gftools.fix import convert_cmap_subtables_to_v4, drop_nonpid0_cmap, drop_mac_cmap, FontFixer

description = "Manipulate a collection of fonts' cmap tables."


def convert_cmap_subtables_to_v4_with_report(font):
  converted = convert_cmap_subtables_to_v4(font)
  for c in converted:
    print(('Converted format {} cmap subtable'
     ' with Platform ID = {} and Encoding ID = {}'
     ' to format 4.').format(c))
  return converted

def main(args=None):
  parser = ArgumentParser(description=description)
  parser.add_argument('fonts', nargs='+')
  parser.add_argument('--format-4-subtables', '-f4', default=False,
                      action='store_true',
                      help="Convert cmap subtables to format 4")
  parser.add_argument('--drop-mac-subtable', '-dm', default=False,
                      action='store_true',
                      help='Drop Mac cmap subtables')
  parser.add_argument('--keep-only-pid-0', '-k0', default=False,
                      action='store_true',
                      help=('Keep only cmap subtables with pid=0'
                            ' and drop the rest.'))
  args = parser.parse_args(args)

  for path in args.fonts:
    fixer = FontFixer(path, verbose=True)
    if args.format_4_subtables:
      print('\nConverting Cmap subtables to format 4...')
      fixer.fixes.append(convert_cmap_subtables_to_v4_with_report)

    if args.keep_only_pid_0:
      print('\nDropping all Cmap subtables,'
            ' except the ones with PlatformId = 0...')
      fixer.fixes.append(drop_nonpid0_cmap)
    elif args.drop_mac_subtable:
      print('\nDropping any Cmap Mac subtable...')
      fixer.fixes.append(drop_mac_cmap)

    fixer.fix()

if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: knowledge.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='knowledge.proto',
  package='fonts',
  syntax='proto2',
  serialized_options=b'\n\026com.google.fonts.protoB\tKnowledge',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0fknowledge.proto\x12\x05\x66onts\"!\n\x0eKnowledgeProto\x12\x0f\n\x07modules\x18\x01 \x03(\t\"N\n\x0bModuleProto\x12\x0f\n\x07\x61lt_ids\x18\x01 \x03(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x65xcerpt\x18\x03 \x01(\t\x12\x0f\n\x07lessons\x18\x04 \x03(\t\"+\n\nTopicProto\x12\x0f\n\x07\x61lt_ids\x18\x01 \x03(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"\xd2\x01\n\x0bLessonProto\x12\x0f\n\x07\x61lt_ids\x18\x01 \x03(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x61uthors\x18\x03 \x03(\t\x12\x11\n\treviewers\x18\x04 \x03(\t\x12\x0e\n\x06topics\x18\x05 \x03(\t\x12\x14\n\x0cprev_lessons\x18\x06 \x03(\t\x12\x14\n\x0cnext_lessons\x18\x07 \x03(\t\x12\x15\n\rrelated_terms\x18\x08 \x03(\t\x12\x0f\n\x07\x65xcerpt\x18\t \x01(\t\x12\x1c\n\x14related_content_urls\x18\n \x03(\t\"r\n\tTermProto\x12\x0f\n\x07\x61lt_ids\x18\x01 \x03(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x65xcerpt\x18\x03 \x01(\t\x12\x17\n\x0frelated_lessons\x18\x04 \x03(\t\x12\x1c\n\x14related_content_urls\x18\x05 \x03(\t\"\x8d\x01\n\x11\x43ontributorsProto\x12?\n\x0c\x63ontributors\x18\x01 \x03(\x0b\x32).fonts.ContributorsProto.ContributorProto\x1a\x37\n\x10\x43ontributorProto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rpersonal_site\x18\x02 \x01(\tB#\n\x16\x63om.google.fonts.protoB\tKnowledge'
)




_KNOWLEDGEPROTO = _descriptor.Descriptor(
  name='KnowledgeProto',
  full_name='fonts.KnowledgeProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='modules', full_name='fonts.KnowledgeProto.modules', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=59,
)


_MODULEPROTO = _descriptor.Descriptor(
  name='ModuleProto',
  full_name='fonts.ModuleProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='alt_ids', full_name='fonts.ModuleProto.alt_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='fonts.ModuleProto.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='excerpt', full_name='fonts.ModuleProto.excerpt', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lessons', full_name='fonts.ModuleProto.lessons', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=61,
  serialized_end=139,
)


_TOPICPROTO = _descriptor.Descriptor(
  name='TopicProto',
  full_name='fonts.TopicProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='alt_ids', full_name='fonts.TopicProto.alt_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='fonts.TopicProto.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=141,
  serialized_end=184,
)


_LESSONPROTO = _descriptor.Descriptor(
  name='LessonProto',
  full_name='fonts.LessonProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='alt_ids', full_name='fonts.LessonProto.alt_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='fonts.LessonProto.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='authors', full_name='fonts.LessonProto.authors', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reviewers', full_name='fonts.LessonProto.reviewers', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='topics', full_name='fonts.LessonProto.topics', index=4,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prev_lessons', full_name='fonts.LessonProto.prev_lessons', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='next_lessons', full_name='fonts.LessonProto.next_lessons', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='related_terms', full_name='fonts.LessonProto.related_terms', index=7,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='excerpt', full_name='fonts.LessonProto.excerpt', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='related_content_urls', full_name='fonts.LessonProto.related_content_urls', index=9,
      number=10, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=187,
  serialized_end=397,
)


_TERMPROTO = _descriptor.Descriptor(
  name='TermProto',
  full_name='fonts.TermProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='alt_ids', full_name='fonts.TermProto.alt_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='fonts.TermProto.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='excerpt', full_name='fonts.TermProto.excerpt', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='related_lessons', full_name='fonts.TermProto.related_lessons', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='related_content_urls', full_name='fonts.TermProto.related_content_urls', index=4,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=399,
  serialized_end=513,
)


_CONTRIBUTORSPROTO_CONTRIBUTORPROTO = _descriptor.Descriptor(
  name='ContributorProto',
  full_name='fonts.ContributorsProto.ContributorProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='fonts.ContributorsProto.ContributorProto.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='personal_site', full_name='fonts.ContributorsProto.ContributorProto.personal_site', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=602,
  serialized_end=657,
)

_CONTRIBUTORSPROTO = _descriptor.Descriptor(
  name='ContributorsProto',
  full_name='fonts.ContributorsProto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='contributors', full_name='fonts.ContributorsProto.contributors', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_CONTRIBUTORSPROTO_CONTRIBUTORPROTO, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=516,
  serialized_end=657,
)

_CONTRIBUTORSPROTO_CONTRIBUTORPROTO.containing_type = _CONTRIBUTORSPROTO
_CONTRIBUTORSPROTO.fields_by_name['contributors'].message_type = _CONTRIBUTORSPROTO_CONTRIBUTORPROTO
DESCRIPTOR.message_types_by_name['KnowledgeProto'] = _KNOWLEDGEPROTO
DESCRIPTOR.message_types_by_name['ModuleProto'] = _MODULEPROTO
DESCRIPTOR.message_types_by_name['TopicProto'] = _TOPICPROTO
DESCRIPTOR.message_types_by_name['LessonProto'] = _LESSONPROTO
DESCRIPTOR.message_types_by_name['TermProto'] = _TERMPROTO
DESCRIPTOR.message_types_by_name['ContributorsProto'] = _CONTRIBUTORSPROTO
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

KnowledgeProto = _reflection.GeneratedProtocolMessageType('KnowledgeProto', (_message.Message,), {
  'DESCRIPTOR' : _KNOWLEDGEPROTO,
  '__module__' : 'knowledge_pb2'
  # @@protoc_insertion_point(class_scope:fonts.KnowledgeProto)
  })
_sym_db.RegisterMessage(KnowledgeProto)

ModuleProto = _reflection.GeneratedProtocolMessageType('ModuleProto', (_message.Message,), {
  'DESCRIPTOR' : _MODULEPROTO,
  '__module__' : 'knowledge_pb2'
  # @@protoc_insertion_point(class_scope:fonts.ModuleProto)
  })
_sym_db.RegisterMessage(ModuleProto)

TopicProto = _reflection.GeneratedProtocolMessageType('TopicProto', (_message.Message,), {
  'DESCRIPTOR' : _TOPICPROTO,
  '__module__' : 'knowledge_pb2'
  # @@protoc_insertion_point(class_scope:fonts.TopicProto)
  })
_sym_db.RegisterMessage(TopicProto)

LessonProto = _reflection.GeneratedProtocolMessageType('LessonProto', (_message.Message,), {
  'DESCRIPTOR' : _LESSONPROTO,
  '__module__' : 'knowledge_pb2'
  # @@protoc_insertion_point(class_scope:fonts.LessonProto)
  })
_sym_db.RegisterMessage(LessonProto)

TermProto = _reflection.GeneratedProtocolMessageType('TermProto', (_message.Message,), {
  'DESCRIPTOR' : _TERMPROTO,
  '__module__' : 'knowledge_pb2'
  # @@protoc_insertion_point(class_scope:fonts.TermProto)
  })
_sym_db.RegisterMessage(TermProto)

ContributorsProto = _reflection.GeneratedProtocolMessageType('ContributorsProto', (_message.Message,), {

  'ContributorProto' : _reflection.GeneratedProtocolMessageType('ContributorProto', (_message.Message,), {
    'DESCRIPTOR' : _CONTRIBUTORSPROTO_CONTRIBUTORPROTO,
    '__module__' : 'knowledge_pb2'
    # @@protoc_insertion_point(class_scope:fonts.ContributorsProto.ContributorProto)
    })
  ,
  'DESCRIPTOR' : _CONTRIBUTORSPROTO,
  '__module__' : 'knowledge_pb2'
  # @@protoc_insertion_point(class_scope:fonts.ContributorsProto)
  })
_sym_db.RegisterMessage(ContributorsProto)
_sym_db.RegisterMessage(ContributorsProto.ContributorProto)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
"""
gftools lang-support

Adds language support metadata to METADATA.pb files.

Usage:

# Standard usage. Does not overwrite existing data.
gftools lang-support -l ./lang/ ./ofl/noto*/METADATA.pb

# Generate a report with insights about data and potential metadata holes.
gftools lang-support -l ./lang/ -r ./ofl/noto*/METADATA.pb

"""

import argparse
from fontTools.ttLib import TTFont
from gflanguages import (LoadLanguages,
                         LoadScripts)
from gftools import fonts_public_pb2
from gftools.util import google_fonts as fonts
from google.protobuf import text_format
from pkg_resources import resource_filename
import csv
import os
from pkg_resources import resource_filename

parser = argparse.ArgumentParser(description='Add language support metadata to METADATA.pb files')
parser.add_argument('--lang', '-l', help='Path to lang metadata package')
parser.add_argument('--report', '-r', action="store_true", help='Whether to output a report of lang metadata insights')
parser.add_argument('--sample_text_audit', '-s', action="store_true", help='Whether to run the sample text audit')
parser.add_argument('--out', '-o', help='Path to output directory for report')
parser.add_argument('metadata_files', help='Path to METADATA.pb files', nargs="+")


def _WriteCsv(path, rows):
  with open(path, 'w', newline='') as csvfile:
    writer = csv.writer(csvfile, delimiter='\t', quotechar='"',
                        quoting=csv.QUOTE_MINIMAL)
    for row in rows:
      writer.writerow(row)


def _WriteReport(metadata_paths, out_dir, languages):
  rows = [[ 'id', 'name', 'lang', 'script', 'population', 'ec_base', 'ec_auxiliary',
           'ec_marks', 'ec_numerals', 'ec_punctuation', 'ec_index', 'st_fallback',
           'st_fallback_name', 'st_masthead_full', 'st_masthead_partial',
           'st_styles', 'st_tester', 'st_poster_sm', 'st_poster_md',
           'st_poster_lg', 'st_specimen_48', 'st_specimen_36', 'st_specimen_32',
           'st_specimen_21', 'st_specimen_16']]

  without_lang = []
  without_sample_text = []
  supported_without_sample_text = {}
  for metadata_path in metadata_paths:
    family = fonts.ReadProto(fonts_public_pb2.FamilyProto(), metadata_path)
    if len(family.languages) == 0:
      without_lang.append(family.name)
    else:
      supports_lang_with_sample_text = False
      for lang_code in family.languages:
        if languages[lang_code].HasField('sample_text'):
          supports_lang_with_sample_text = True
          break
      if not supports_lang_with_sample_text:
        without_sample_text.append(family.name)
    for l in family.languages:
      if not languages[l].HasField('sample_text') and l not in supported_without_sample_text:
        supported_without_sample_text[l] = languages[l]

  for lang in supported_without_sample_text.values():
    rows.append([lang.id, lang.name, lang.language, lang.script, lang.population])

  path = os.path.join(out_dir, 'support.csv')
  _WriteCsv(path, rows)


def _SampleTextAudit(out_dir, languages, scripts, unused_scripts=[]):
  rows = [['id','language','script','has_sample_text','historical']]
  # sort by script|has_sample_text|historical|id
  entries = []

  min_sample_text_languages = 0
  by_script = {}
  for l in languages.values():
    if l.script not in by_script:
      by_script[l.script] = []
    by_script[l.script].append(l)
  for script in by_script:
    if script in unused_scripts:
      continue
    languages_with_sample_text = {l.id for l in by_script[script] if l.HasField('sample_text') and not l.sample_text.HasField('fallback_language')}
    non_historical_languages_without_sample_text = [l for l in by_script[script] if not l.historical and l.id not in languages_with_sample_text]
    if len(languages_with_sample_text) < 2:
      if len(languages_with_sample_text) == 1 and len(by_script[script]) > 1 and len(non_historical_languages_without_sample_text) > 1:
        min_sample_text_languages += 1
      elif len(languages_with_sample_text) == 0:
        if len(non_historical_languages_without_sample_text) > 1:
          min_sample_text_languages += 2
        else:
          min_sample_text_languages += 1

    if len(languages_with_sample_text) == 0 or (len(languages_with_sample_text) == 1 and len([l for l in by_script[script] if not l.historical]) > 1 ):
      for l in by_script[script]:
        entries.append({
          'id': l.id,
          'language': l.name,
          'script': scripts[l.script].name,
          'has_sample_text': l.id in languages_with_sample_text,
          'historical': l.historical,
        })

  print(min_sample_text_languages)

  last_script = None
  entries.sort(key = lambda x: (x['script'], not x['has_sample_text'], not x['historical'], x['id']))
  for e in entries:
    if last_script is not None and e['script'] != last_script:
      rows.append([])
    rows.append([e['id'], e['language'], e['script'], 'X' if e['has_sample_text'] else '', 'X' if e['historical'] else ''])
    last_script = e['script']

  path = os.path.join(out_dir, 'sample_text_audit.csv')
  _WriteCsv(path, rows)


def main(args=None):
  args = parser.parse_args(args)
  languages = LoadLanguages(base_dir=args.lang)
  scripts = LoadScripts(base_dir=args.lang)

  if args.report:
    assert len(argv) > 1, 'No METADATA.pb files specified'
    assert args.out is not None, 'No output dir specified (--out)'
    print('Writing insights report...')
    _WriteReport(argv[1:], args.out, languages)
  elif args.sample_text_audit:
    assert args.out is not None, 'No output dir specified (--out)'
    print('Auditing sample text')
    seen_scripts = set()
    unused_scripts = set()
    for path in argv[1:]:
      family = fonts.ReadProto(fonts_public_pb2.FamilyProto(), path)
      for l in family.languages:
        seen_scripts.add(languages[l].script)
    for s in scripts:
      if s not in seen_scripts:
        unused_scripts.add(s)
    _SampleTextAudit(args.out, languages, scripts, unused_scripts)
  else:
    language_comments = fonts.LanguageComments(languages)
    for path in args.metadata_files:
      family_metadata = fonts.ReadProto(fonts_public_pb2.FamilyProto(), path)
      if len(family_metadata.languages) > 0:
        continue
      exemplar_font_fp = os.path.join(
        os.path.dirname(path), fonts.GetExemplarFont(family_metadata).filename
      )
      exemplar_font = TTFont(exemplar_font_fp)
      supported_languages = fonts.SupportedLanguages(exemplar_font, languages)
      supported_languages = sorted([l.id for l in supported_languages])
      family_metadata.languages.extend(supported_languages)
      fonts.WriteProto(family_metadata, path, comments=language_comments)



if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-

inputFilePath = "/Path/To/File/GlyphData.txt"

import sys, os

if len(sys.argv) > 1:
	inputFilePath = sys.argv[1]

if len(inputFilePath) < 5:
	print "Please supply a path to a file. This can be either a GlyphData.xml or a tab separated .txt file"
	exit()

if not os.path.isfile(inputFilePath):
	print "Could not find file at path:", inputFilePath
	exit()

# This list defineds the columns and ordering in the tabbed file.
fields = ["unicode",
		"name",
		#"unicode2",
		"sortName",
		"sortNameKeep",
		"decompose",
		"category",
		"subCategory",
		"script",
		"production",
		"altNames",
		"description",
		"anchors",
		"accents"]

def writeTabbedHeader(File):
	line = "\t".join(fields)+"\n"
	File.write(line)
	
def writeTabbedContent(inputFilePath, outFile):
	import xml.etree.ElementTree
	tree = xml.etree.ElementTree.parse(inputFilePath).getroot()
	i = 0
	for node in tree.iter():
		if node.tag != "glyph":
			continue
		nodeAttribs = dict(node.attrib)
		attribs = []
		for field in fields:
			attrib = nodeAttribs.get(field, "")
			if field == "unicode" and len(attrib) > 0:
				attrib = "\"%s\"" % attrib # Excel and Number import is as numbers anyway but you got to try
			attribs.append(attrib)
			try:
				nodeAttribs.pop(field)
			except:
				pass
		if len(nodeAttribs) > 0:
			print "Attributes not written:", node.attrib["name"], nodeAttribs
		line = "\t".join(attribs)+"\n"
		outFile.write(line.encode("utf-8"))

def writeDataHeader(File):
	File.write('<?xml version="1.0" encoding="UTF-8" ?>\n\
<!DOCTYPE glyphData [\n\
<!ELEMENT glyphData (glyph)+>\n\
<!ELEMENT glyph EMPTY>\n\
<!ATTLIST glyph\n\
	unicode			CDATA		#IMPLIED\n\
	name			CDATA		#REQUIRED\n\
	sortName		CDATA		#IMPLIED\n\
	sortNameKeep	CDATA		#IMPLIED\n\
	category		CDATA		#REQUIRED\n\
	subCategory		CDATA		#IMPLIED\n\
	script			CDATA		#IMPLIED\n\
	description		CDATA		#REQUIRED\n\
	production		CDATA		#IMPLIED\n\
	altNames		CDATA		#IMPLIED\n\
	decompose		CDATA		#IMPLIED\n\
	anchors			CDATA		#IMPLIED\n\
	accents			CDATA		#IMPLIED>\n\
]>\n\
<glyphData>\n')

def writeDataContent(inputFilePath, outFile):
	import codecs
	inFile = codecs.open(inputFilePath, "r", "utf-8")
	
	fields = inFile.readline().strip('\n').split("\t")
	print "Reading Fields:",", ".join(fields)
	for line in inFile:
		attribs = line.strip('\n').split("\t")
		line = u'	<glyph '
		for idx, field in enumerate(fields):
			attrib = attribs[idx]
			if len(attrib) > 0:
				attrib = attrib.strip("\"")
				line += u'%s="%s" ' % (field, attrib)
		line += '/>\n'
		outFile.write(line.encode("utf-8"))
	outFile.write('</glyphData>\n')
	inFile.close()

if inputFilePath[-4:] == ".xml":
	print "To Tabbed", inputFilePath
	outputFilePath = inputFilePath[:-4]+".txt"
	outputFile = open(outputFilePath, "w")
	writeTabbedHeader(outputFile)
	writeTabbedContent(inputFilePath, outputFile)
	outputFile.close()
elif inputFilePath[-4:] == ".txt":
	print "To XML: ", inputFilePath
	outputFilePath = inputFilePath[:-4]+"_new.xml"
	outputFile = open(outputFilePath, "w")
	writeDataHeader(outputFile)
	writeDataContent(inputFilePath, outputFile)
	outputFile.close()
	
	



#!/usr/bin/env python3
# Copyright 2016 The Fontbakery Authors
# Copyright 2017 The Google Font Tools Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Unittests to check the functionality of Google Fonts Tools"""
import os
import re
from glob import glob
import unittest
import subprocess
from gftools.scripts import _get_subcommands


class TestGFToolsScripts(unittest.TestCase):
    """Functional tests to determine whether each script can execute successfully"""
    def setUp(self):
        self.example_dir = os.path.join('data', 'test', 'cabin')
        self.example_font = os.path.join(self.example_dir, 'Cabin-Regular.ttf')
        self.example_family = glob(os.path.join("data", "test", "mavenpro", "*.ttf"))
        self.example_vf_font = os.path.join("data", "test", 'Lora-Roman-VF.ttf')
        self.example_vf_stat = os.path.join("data", "test", 'lora_stat.yaml')
        self.example_glyphs_file = os.path.join("data", "test", 'Lora.glyphs')
        self.example_builder_config = os.path.join("data", "test", 'builder_test.yaml')
        self.example_builder_config_2_sources = os.path.join("data", "test", "Libre-Bodoni", "sources", "config.yaml")
        self.src_vtt_font = os.path.join("data", "test", "Inconsolata[wdth,wght].ttf")
        self.gf_family_dir = os.path.join('data', 'test', 'mock_googlefonts', 'ofl', 'abel')
        self.nam_file = os.path.join('data', 'test', 'arabic_unique-glyphs.nam')
        self.dir_before_tests = os.listdir(self.example_dir)

    def tearDown(self):
        """Clears the example folder of any files created during the unit tests"""
        files_to_delete = set(os.listdir(self.example_dir)) - set(self.dir_before_tests)
        for f in files_to_delete:
            os.remove(os.path.join(self.example_dir, f))

    def test_add_font(self):
        from gftools.scripts.add_font import main
        main([self.gf_family_dir])

    def test_build_ofl(self):
        from gftools.scripts.build_ofl import main
        main([self.example_dir])

    def test_check_bbox(self):
        from gftools.scripts.check_bbox import main
        main([self.example_font, '--glyphs', '--extremes'])

    def test_check_copyright_notices(self):
        from gftools.scripts.check_copyright_notices import main
        main([self.example_font])

    def test_check_font_version(self):
        from gftools.scripts.check_font_version import main
        main(["Cabin"])

    def test_check_name(self):
        from gftools.scripts.check_name import main
        main([self.example_font])

    def test_check_vtt_compatibility(self):
        from gftools.scripts.check_vtt_compatibility import main
        main([self.example_font, self.example_font])

    def test_compare_font(self):
        from gftools.scripts.compare_font import main
        main([self.example_font, self.example_font])

    def test_find_features(self):
        from gftools.scripts.find_features import main
        main([self.example_font])

    def test_fix_ascii_fontmetadata(self):
        from gftools.scripts.fix_ascii_fontmetadata import main
        main([self.example_font])

    def test_fix_cmap(self):
        from gftools.scripts.fix_cmap import main
        main([self.example_font])

    def test_fix_familymetadata(self):
        from gftools.scripts.fix_familymetadata import main
        main([self.example_font])

    def test_fix_fsselection(self):
        from gftools.scripts.fix_fsselection import main
        main([self.example_font])

    def test_fix_fstype(self):
        from gftools.scripts.fix_fstype import main
        main([self.example_font])

    def test_fix_gasp(self):
        from gftools.scripts.fix_gasp import main
        main([self.example_font])

    def test_fix_glyph_private_encoding(self):
        from gftools.scripts.fix_glyph_private_encoding import main
        main([self.example_font])

    def test_fix_glyphs(self):
        from gftools.scripts.fix_glyphs import main
        main([self.example_glyphs_file])

    def test_fix_hinting(self):
        from gftools.scripts.fix_hinting import main
        main([self.example_font])

    def test_fix_isfixedpitch(self):
        from gftools.scripts.fix_isfixedpitch import main
        main(["--fonts", self.example_font])

    def test_fix_nameids(self):
        from gftools.scripts.fix_nameids import main
        main([self.example_font])

    def test_fix_nonhinting(self):
        from gftools.scripts.fix_nonhinting import main
        main([self.example_font, self.example_font + '.fix'])

    def test_fix_ttfautohint(self):
        from gftools.scripts.fix_ttfautohint import main
        main([self.example_font])

    def test_fix_vendorid(self):
        from gftools.scripts.fix_vendorid import main
        main([self.example_font])

    def test_fix_vertical_metrics(self):
        from gftools.scripts.fix_vertical_metrics import main
        main([self.example_font])

    def test_font_diff(self):
        from gftools.scripts.font_diff import main
        main([self.example_font, self.example_font])

    def test_font_weights_coverage(self):
        from gftools.scripts.font_weights_coverage import main
        main([self.example_dir])

    def test_fix_font(self):
        from gftools.scripts.fix_font import main
        main([self.example_font])

    def test_fix_family(self):
        from gftools.scripts.fix_family import main
        main(self.example_family)

    def test_list_italicangle(self):
        from gftools.scripts.list_italicangle import main
        main([self.example_font])

    def test_list_panose(self):
        from gftools.scripts.list_panose import main
        main([self.example_font])

    # def test_list_variable_source(self):
    #     from gftools.scripts.list_variable_source import main

    def test_list_weightclass(self):
        from gftools.scripts.list_weightclass import main
        main([self.example_font])

    def test_list_widthclass(self):
        from gftools.scripts.list_widthclass import main
        main([self.example_font])

    def test_nametable_from_filename(self):
        from gftools.scripts.nametable_from_filename import main
        main([self.example_font])

    def test_namelist(self):
        from gftools.scripts.namelist import main
        main([self.example_font])

    def test_ots(self):
        from gftools.scripts.ots import main
        main([self.example_dir])

    def test_rangify(self):
        from gftools.scripts.rangify import main
        main([self.nam_file])

    def test_test_gf_coverage(self):
        from gftools.scripts.test_gf_coverage import main
        main([self.example_font])

    def test_ttf2cp(self):
        from gftools.scripts.ttf2cp import main
        main([self.example_font])

    def test_unicode_names(self):
        from gftools.scripts.unicode_names import main
        main(["--nam_file", self.nam_file])

    def test_update_families(self):
        from gftools.scripts.update_families import main
        main([self.example_font])

    def test_update_version(self):
        from gftools.scripts.update_version import main
        main(["--old_version", "2.00099", "--new_version", "2.001", self.example_font])

    def test_varfont_info(self):
        from gftools.scripts.varfont_info import main
        main([self.example_vf_font])

    def test_what_subsets(self):
        from gftools.scripts.what_subsets import main
        main([self.example_font])

#     def test_rename_font(self):
#         from gftools.scripts.rename-font'), self.example_font, "Foobar"])
# # Temporarily disabling this until we close issue #13
# # (https://github.com/googlefonts/tools/issues/13)
# # See also https://github.com/googlefonts/fontbakery/issues/1535
# #    def test_update_families(self):
# #        from gftools.scripts.update-families'), self.example_font])

#     def test_update_nameids(self):
#         from gftools.scripts.update-nameids'), self.example_font, "-c", "Foobar"])

#     def test_check_vtt_compile(self):
#         from gftools.scripts.check-vtt-compile'), self.src_vtt_font])

#     def test_gen_stat(self):
#         self.check_script(
#             ['python', self.get_path('gen-stat'), self.example_vf_font, "--axis-order", "wght"]
#         )

#     def test_gen_stat2(self):
#         self.check_script(
#             ['python', self.get_path('gen-stat'), self.example_vf_font, "--src", self.example_vf_stat]
#         )

#     def test_builder(self):
#         from gftools.scripts.builder'), self.example_builder_config])
    
#     def test_builder_2_sources(self):
#         self.check_script(["python", self.get_path("builder"), self.example_builder_config_2_sources])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
gftools fix-family

Update a family so it conforms to the Google Fonts specification
https://github.com/googlefonts/gf-docs/tree/main/Spec

Usage:

gftools fix-family fonts1.ttf fonts2.ttf

# Fix font issues that should be fixed in the source files
gftools fix-family fonts1.ttf --include-source-fixes
"""
import argparse
import logging
import os
from fontTools.ttLib import TTFont
from gftools.fix import *
from gftools.utils import parse_axis_dflts

logging.basicConfig(level=logging.INFO)


def new_filename(font, font_renamed=None):
    if font_renamed:
        return fix_filename(font)
    return os.path.basename(font.reader.file.name)


def main(args=None):
    parser = argparse.ArgumentParser()
    parser.add_argument("fonts", nargs="+", help="Font family paths")
    parser.add_argument(
        "--inplace", action="store_true", default=False, help="Save fixed fonts inplace"
    )
    parser.add_argument("-o", "--out", help="Output dir for fixed fonts")
    parser.add_argument(
        "--include-source-fixes",
        action="store_true",
        help="Fix font issues that should be fixed in the source files.",
    )
    parser.add_argument(
        "--rename-family",
        help="Change the family's name"
    )
    parser.add_argument(
        "--fvar-instance-axis-dflts",
        help=(
            "Set the fvar instance default values for non-wght axes. e.g "
            "wdth=100 opsz=36"
        )
    )
    args = parser.parse_args(args)

    fonts = [TTFont(f) for f in args.fonts]
    if args.fvar_instance_axis_dflts:
        axis_dflts = parse_axis_dflts(args.fvar_instance_axis_dflts)
    else:
        axis_dflts = None
    fonts = fix_family(fonts, args.include_source_fixes, args.rename_family, axis_dflts)

    if args.inplace:
        for font in fonts:
            font.save(font.reader.file.name)
    elif args.out:
        if not os.path.isdir(args.out):
            os.mkdir(args.out)
        for font in fonts:
            filename = new_filename(font, args.rename_family)
            out_path = os.path.join(args.out, filename)
            font.save(out_path)
    else:
        for font in fonts:
            dir_ = os.path.dirname(font.reader.file.name)
            filename = os.path.join(dir_, new_filename(font, args.rename_family))
            if filename == font.reader.file.name:
                font.save(filename + ".fix")
            else:
                font.save(filename)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""Create or author Google Fonts axisregistry {AXIS_NAME}.textproto files."""
import sys
import argparse
from gftools.axes_pb2 import AxisProto, FallbackProto
from google.protobuf import text_format
from fontTools.ttLib import TTFont


parser = argparse.ArgumentParser(
    prog='gftools add-axis',
    description=__doc__,
    )

parser.add_argument(
            'font',
            type=str,
            help='The font file to the axis values from.')


class ProgramAbortError(Exception):
    pass


class UserAbortError(Exception):
    pass


def _get_fvar_axis(name_table, fvar_table):
    axes = []
    for axis in fvar_table.axes:
        axes.append((axis, f'{name_table.getName(axis.axisNameID, 3, 1, 0x0409)} {axis.axisTag}'))
    axes.sort(key=lambda a:a[0].axisTag)
    choices = '\n'.join([f'  {index}: {label}' for index, (_, label) in enumerate(axes)])
    question = ('Found axes:\n'
                f'{choices}'
                '\n'
                'pick one by number (e.g. 0), q=quit:')
    while True:
        try:
            answer = input(question).strip()
            if answer == 'q':
                raise UserAbortError()
            index = int(answer)  # raises ValueError
            fvar_axis, _ = axes[index]  # raises IndexError
        except (ValueError, IndexError):
            # must try again
            continue
        print(f'You picked: {fvar_axis.axisTag}.')
        return fvar_axis


def _get_fallbacks_gen(name_table, stat_axis_index, AxisValue):
    for stat_axis_value in AxisValue:

        if stat_axis_value.Format in (1, 3):
            if stat_axis_value.AxisIndex == stat_axis_index:
                yield (
                    name_table.getName(stat_axis_value.ValueNameID, 3, 1, 0x0409),
                    stat_axis_value.Value
                )
        elif stat_axis_value.Format == 4:
            for avr in stat_axis_value.AxisValueRecord:
                if avr.AxisIndex == stat_axis_index:
                    yield (
                        name_table.getName(stat_axis_value.ValueNameID, 3, 1, 0x0409),
                        avr.Value
                    )
        else:
            print(f'SKIP STAT AxisValue can\'t handel Format {stat_axis_value.Format} '
                  f'({name_table.getName(stat_axis_value.ValueNameID, 3, 1, 0x0409)})')


def add_axis(font: str):
    axis_proto = AxisProto()
    ttFont = TTFont(font)
    name_table = ttFont['name']
    try:
        fvar_table = ttFont['fvar']
    except KeyError:
        raise ProgramAbortError('No fvar present')
    fvar_axis = _get_fvar_axis(name_table, fvar_table)

    # Axis tag
    axis_proto.tag = fvar_axis.axisTag
    # Display name for axis, e.g. "Optical size" for 'opsz'
    # Like 'Name' in
    # https://docs.microsoft.com/en-us/typography/opentype/spec/dvaraxistag_opsz
    # name_table.getName(
    #           NameID,
    #           <Platform ID: 3 = Windows>,
    #           <encodingID, Platform-specific encoding ID: 1 = Unicode BMP>,
    #           <Language ID: 0x0409 = 1033 = en_us>)
    axis_proto.display_name = \
        f'{name_table.getName(fvar_axis.axisNameID, 3, 1, 0x0409)}'
    # Lower bound for the axis
    axis_proto.min_value = fvar_axis.minValue
    # The default position to use and to prefer for exemplars
    axis_proto.default_value = fvar_axis.defaultValue
    # Upper bound for the axis
    axis_proto.max_value = fvar_axis.maxValue
    # Input values for this axis must aligned to 10^precision
    axis_proto.precision = 1  # ask user?
    # Short descriptive paragraph
    axis_proto.description = (  # ask user?
        'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod'
        ' tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim'
        ' veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea'
        ' commodo consequat. Duis aute irure dolor in reprehenderit in voluptate'
        ' velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint'
        ' occaecat cupidatat non proident, sunt in culpa qui officia deserunt'
        ' mollit anim id est laborum.'
        )

    fallback_proto = FallbackProto()
    fallback_proto.name = 'Default'
    fallback_proto.value = fvar_axis.defaultValue
    axis_proto.fallback.append(fallback_proto)

    text_proto = text_format.MessageToString(axis_proto, as_utf8=True,
                                             use_index_order=True)
    filename = f'{axis_proto.display_name.lower()}.textproto'
    with open(filename, 'x') as f:
        f.write(text_proto)
    print(f'DONE create {filename}!')


def main(args=None):
    try:
        args = parser.parse_args(args)
        add_axis(args.font)
    except UserAbortError:
        print('Aborted by user!')
        sys.exit(1)
    except ProgramAbortError as e:
        print(f'Aborted by program: {e}')
        sys.exit(1)


if __name__ == '__main__':
    main()



"""Generate STAT tables for a Variable Font Family

This module exports the functions "gen_stat_tables" and
"gen_stat_tables_from_config" which can be used to
generate a STAT table for each font in a Variable Font Family.

The STAT AxisValues are constructed using the Google Font's Axis Registry,
https://github.com/google/fonts/tree/main/axisregistry

The function should be able to make STAT tables for any family with the
following properties:
- All fonts contain the same amount of fvar axes
- All fvar axes have the same ranges
"""
from axisregistry import AxisRegistry
from fontTools.otlLib.builder import buildStatTable
from gftools.utils import font_is_italic
import os
import logging


__all__ = ["gen_stat_tables", "gen_stat_tables_from_config"]


log = logging.getLogger(__name__)


def gen_stat_tables(ttFonts):
  from axisregistry import build_stat
  for ttFont in ttFonts:
    siblings = [f for f in ttFonts if f != ttFont]
    build_stat(ttFont, siblings)


def gen_stat_tables_from_config(stat, varfonts, has_italic=None, locations=None):
    """
    Generate a stat table for each font in a family from a Python configuration.

    Args:
        stat: either a dictionary or list as described below
        varfonts: a list of variable TTFont instances
        has_italic: a boolean indicating whether the family contains an italic.
            If not provided, the stylename of the font files are inspected to
            determine if any of them contain the word ``Italic``.

    The ``stat`` parameter should normally be a list of axis dictionaries in the
    format used by ``buildStatTable``. This list should *not* contain an entry
    for the ``ital`` axis, as this entry will be generated as appropriate for
    each font if ``has_italic`` is True.

    For example::

        varfonts = [
            "Source-Regular-VF[wdth].ttf",
            "Source-Italic-VF[wdth].ttf"
        ]
        stat = [
                { "tag":"wdth", "name": "Width", "values": [ ... ] }
        ]

    Alternately, to allow different STAT table entries for each font, the ``stat``
    parameter may be a dictionary, whose keys are source IDs (usually source
    filenames) corresponding to the appropriate entry in the ``varfonts``
    dictionary and whose values are the list of axis dictionaries for the font.
    Note that in this case, the axes list is passed to ``buildStatTable`` with
    no further manipulation, meaning that if you want an ``ital`` axis, you
    should specify it manually as part of the dictionary.

    For example::

        stat = {
            "Font[wght].ttf": [
                { "tag":"wdth", "name": "Width", "values": [ ... ] },
                { "tag":"ital", "name": "Italic", "values": [ ... ] }
            ],
            "Font-Italic[wght].ttf": [
                { "tag":"wdth", "name": "Width", "values": [ ... ] },
                { "tag":"ital", "name": "Italic", "values": [ ... ] }
            ]
        }
    """
    assert all("fvar" in f for f in varfonts)
    # Check we have no italic
    if isinstance(stat, list):
        if has_italic is None:
            has_italic = any(font_is_italic(f) for f in varfonts)
        if has_italic:
            for ax in stat:
                if ax["name"] == "ital":
                    raise ValueError("ital axis should not appear in stat config")
            ital_stat_for_roman = {
                "name": "Italic", "tag": "ital",
                "values": [dict(value=0, name="Roman", flags=0x2, linkedValue=1)]
            }
            ital_stat_for_italic = {
                "name": "Italic", "tag": "ital",
                "values": [dict(value=1, name="Italic")]
            }

            stat.append({})  # We will switch this entry between Roman and Italic

    for ttFont in varfonts:
        filename = os.path.basename(ttFont.reader.file.name)
        if isinstance(stat, dict):
            if filename not in stat:
                raise ValueError("Filename %s not found in stat dictionary" % filename)
            this_stat = stat[filename]
        else:
            if has_italic:
                if font_is_italic(ttFont):
                    stat[-1] = ital_stat_for_italic
                else:
                    stat[-1] = ital_stat_for_roman
            this_stat = stat

        if isinstance(locations, dict):
            if filename not in locations:
                raise ValueError("Filename %s not found in locations" % filename)
            locations = locations[filename]
        buildStatTable(ttFont, this_stat, locations=locations)



"""Generates an upstream.yaml from a config.yaml and a GitHub release URL

"""
import argparse
import os
from tempfile import TemporaryDirectory
import yaml
import zipfile

import gftools.packager
from gftools.builder import GFBuilder
from strictyaml import as_document
from gftools.utils import download_file
from fontTools.ttLib import TTFont


parser = argparse.ArgumentParser(description="Create an upstream.yaml for a family")
parser.add_argument("url", help="URL of GitHub release")
parser.add_argument("--family", help="Family name", required=False)
parser.add_argument("--config", help="Config file", default="sources/config.yaml", required=False)


def get_family_name(config):
    if "familyName" in config:
        return config["familyName"]
    return GFBuilder(config).get_family_name()


def generate_upstream(config, url):
    repo = os.environ.get("GITHUB_REPOSITORY")
    if not repo:
        raise ValueError("Not being run from a GitHub action?")
    if "category" not in config:
        config["category"] = ["SANS_SERIF"]

    upstream = {
        "name": get_family_name(config),
        "repository_url": os.environ["GITHUB_SERVER_URL"] + "/" + repo + ".git",
        "archive": url,
        "branch": "main",
        "category": config["category"],
        "build": "",
        "files": {},
        "designer": "Will be filled in",
    }
    return upstream


def update_file_list(upstream):
    print("Downloading release archive...")
    upstream["files"] = {}
    with TemporaryDirectory() as tmp:
        archive_path = os.path.join(tmp, "archive.zip")
        download_file(upstream["archive"], archive_path)
        license_found = False
        description_found = False
        a_font = None
        with zipfile.ZipFile(archive_path, "r") as zip_ref:
            zip_ref.extractall(tmp)
        for root, _, files in os.walk(tmp):
            for file in files:
                fullpath = os.path.join(root, file)
                relpath = os.path.relpath(fullpath, tmp)
                print(relpath)
                if file == "OFL.txt":
                    license_found = True
                    upstream["files"][relpath] = file
                elif file == "DESCRIPTION.en_us.html":
                    description_found = True
                    upstream["files"][relpath] = file
                elif file == "ARTICLE.en_us.html":
                    upstream["files"][relpath] = "article/"+file
                elif file.endswith("ttf"):
                    if config.get("buildVariable", True):
                        # Only add the file if it is the variable font
                        if "[" in file:
                            upstream["files"][relpath] = file
                            a_font = fullpath
                    else:
                        # Add statics
                        upstream["files"][relpath] = file
                        a_font = fullpath

        # If there was a "googlefonts/" directory in the release, just
        # use files in that directory.
        if any("googlefonts/" in x for x in upstream["files"].keys()):
            upstream["files"] = {str(k):str(v) for k,v in upstream["files"].items() if "googlefonts/" in str(k) or not ".ttf" in str(k) }

        if not license_found:
            raise ValueError(
                "No license file was found. Ensure OFL.txt is added the the release"
            )
        if not description_found and "Noto" not in upstream["name"]:
            raise ValueError(
                "No description file was found. Ensure DESCRIPTION.en_us.html is added the the release"
            )
        if not a_font:
            if config.get("buildVariable", True):
                raise ValueError("No variable font files were found. Is the build broken?")
            raise ValueError("No font files were found. Is the release broken?")

        designer = TTFont(a_font)["name"].getDebugName(9)
        if "Noto" in upstream["name"]:
            upstream["designer"] = "Google"
        elif designer:
            upstream["designer"] = designer


if __name__ == "__main__":
    args = parser.parse_args()
    if args.family:
        config = {"familyName": args.family}
    else:
        config = yaml.load(
            open(args.config), Loader=yaml.FullLoader
        )

    if os.path.isfile("upstream.yaml"):
        try:
            upstream = gftools.packager._upstream_conf_from_file(
                "upstream.yaml", yes=True, quiet=True
            )
        except Exception as e:
            raise ValueError("Something went wrong parsing upstream.yaml: " + str(e))
    else:
        try:
            upstream = as_document(
                generate_upstream(config, args.url),
                gftools.packager.upstream_yaml_schema,
            )
        except Exception as e:
            raise ValueError(
                "Something went wrong generating upstream.yaml (bug in updateupstream): "
                + str(e)
            )

    # Add archive URL
    upstream["archive"] = args.url
    update_file_list(upstream)

    with open("upstream.yaml", "w") as upstream_yaml_file:
        upstream_yaml_file.write(upstream.as_yaml())



#!/usr/bin/env python3
# Copyright 2016 The Fontbakery Authors
# Copyright 2017 The Google Fonts Tools Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import print_function
import argparse
import os
import sys
from glyphsets.codepoints import (CodepointsInFont,
                                  CodepointsInNamelist,
                                  nam_dir)


NAM_FILES = [f for f in os.listdir(nam_dir)
             if f.endswith(".nam")]


def main(args=None):
  parser = argparse.ArgumentParser(
    description='Reports the missing codepoints in a font')
  parser.add_argument('font')
  args = parser.parse_args(args)

  if args.font[-4:] != ".ttf":
    sys.exit('Usage: {} fontfile.ttf'.format(sys.argv[0]))

  expected = set()
  for nam_file in NAM_FILES:
    nam_filepath = os.path.join(nam_dir, nam_file)
    expected.update(CodepointsInNamelist(nam_filepath))

  diff = expected - CodepointsInFont(args.font)

  print(args.font),
  if bool(diff):
    print('missing'),
    for c in sorted(diff):
      print('0x%04X' % (c)),
  else:
    print('OK')


if __name__ == '__main__':
  main()



#!/usr/bin/env python3
# Copyright 2013 The Font Bakery Authors.
# Copyright 2017 The Google Fonts Tools Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# See AUTHORS.txt for the list of Authors and LICENSE.txt for the License.
#
import argparse
import os
from fontTools import ttLib
from gftools.fix import fix_pua, FontFixer
from gftools.utils import get_unencoded_glyphs


description = 'Fixes TTF unencoded glyphs to have Private Use Area encodings'

parser = argparse.ArgumentParser(description=description)
parser.add_argument('ttf_font', nargs='+',
                    help='Font in OpenType (TTF/OTF) format')
parser.add_argument('--autofix', action="store_true",
                    help='Apply autofix. '
                         'Otherwise just check if there are unencoded glyphs')


def main(args=None):
  args = parser.parse_args(args)
  for path in args.ttf_font:
    if not os.path.exists(path):
      continue

    if args.autofix:
      FontFixer(path, fixes=[fix_pua], verbose=True).fix()
    else:
      font = ttLib.TTFont(path, 0)
      print(("\nThese are the unencoded glyphs in font file '{0}':\n"
             "{1}").format(path, '\n'.join(get_unencoded_glyphs(font))))

if __name__ == '__main__':
  main()




#!/usr/bin/env python3
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS-IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
A Python script to set a font's OS/2 usWeightClass value so it conforms to
the Google Fonts specification. The font's style name in the name record is
used to determine the correct value.
"""
from __future__ import print_function
from gftools.fix import fix_weight_class, FontFixer
import sys


def main(font_path):
    FontFixer(font_path, verbose=True, fixes=[fix_weight_class]).fix()

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Please include a path to a font")
    else:
        main(sys.argv[1])

